function output = FISTA_Forest(b, RR, xf, input)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% minimize beta*(||Phi*X||_{2,1}+ ||Phi*X||_{Forest}) + 0.5*sum_{s}||R_{s}X(:,s)-b_{s}||_2^2
%%% Phi:DWT,  Phi': IDWT
%%% Jan. 16, 2013, Written by Chen Chen at University of Texas at
%%% Arlington 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parsin.MAXITER=1; parsin.tv='iso'; l=input.l; u=input.u;
if((l==-Inf)&&(u==Inf))
    project=@(x)x;
elseif (isfinite(l)&&(u==Inf))
    project=@(x)(((l<x).*x)+(l*(x<=l)));
elseif (isfinite(u)&&(l==-Inf))
     project=@(x)(((x<u).*x)+((x>=u)*u));
elseif ((isfinite(u)&&isfinite(l))&&(l<u))
    project=@(x)(((l<x)&(x<u)).*x)+((x>=u)*u)+(l*(x<=l));
else
    error('lower and upper bound l,u should satisfy l<u');
end

if iscell(b)
    T=length(b); 
else
    T = 1;
    RR = {RR};
    b = {b};
end

Lx=input.L; 
n1=input.n1; n2=input.n2; N=n1*n2;
alpha = input.alpha; beta = input.beta; lambda = input.lamda;
Phi=input.Phi;

for t=1:T,
    Atb{t}=RR{t}'*b{t};
end

G=input.G;%GtG=input.GtG; 
groups=input.groups; 
Gmat=input.Gmat;
GtG=G'*G;

TV_iso = true; TV_eps = 1e-10;
y=zeros(input.n1*input.n2,T);
f_prev=inf;f=inf; 
tnew=1; yr=zeros(size(y));
oitr = 1;  tnew=1;

iterno=0;
t00 = cputime; t0=t00;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% main loop
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for itr = 1:input.maxitr  % total iter counter        
    iterno=iterno+1; 
    told=tnew;
    yp=y;
    
    %update z
    if itr==1
       z = zeros(size(G,1),1,T);
    else
        z=shrink_group_MT(G,xg,beta/lambda,0,groups,Gmat,T);
    end  
    
    %update yg    
    for t=1:T
        temp=RR{t}'*(RR{t}*yr(:,t));
        temp=temp-Atb{t};
        yg(:,t)=yr(:,t)-temp/Lx; 
        temp2 = Phi'* (GtG*(Phi* yr(:,t)) );
        temp2 =temp2 - Phi'*(G'*z(:,t));
        yg(:,t) = yg(:,t) -(lambda/Lx)*temp2;
    end
    
 
    for t=1:T,
        xg(:,t)=Phi*yg(:,t);
    end
    
      temp = max(0,1-(beta/Lx)./sqrt(sum(xg.^2,2)));
      x=repmat(temp, [1, T]).*xg;
    
     for t=1:T,
         y(:,t)=Phi'*x(:,t);         
     end    

     y = project(y);

    tnew=(1+sqrt(1+4*told^2))/2; 
    yr=y+((told-1)/tnew)*(y-yp);    
%     if itr>30 tnew=1; end
     for  t=1:T
        SNR(1,t) = snr(y(:,t), xf(:,t));
    end

    output.snr(iterno)=snr(y(:), xf(:));

    output.xtime(iterno)=cputime-t00; 
   
     if itr>=input.no  
        break
     end
end

for  t=1:T
        SNR(1,t) = snr(y(:,t), xf(:,t));
end
output.finalsnr= SNR;
output.y=y; %output.x=z;


     function [f, f_prev]=get_f(RR, y, b, alpha, beta, f, T)
        v1 = alpha*TV_MT(y);
        
        temp=zeros(size(y));
        for t=1:T,
            temp(:,t)=Phi'*y(:,t);            
        end
        temp=sqrt(sum(temp.*temp, 2));
        v2 = beta*sum(temp);
          
        v3=0;
        for t=1:T,
            r3 = RR{t}*y(:,t) - b{t}; 
            v3 = v3+0.5*(norm(r3).^2);
        end
        f_prev = f;
        f = v1 + v2 + v3;
     end
   
    function p=Grad1(u)
        % backward finite difference along dim 1
        u = reshape(u,n1,n2);
        p = [u(1,:);diff(u,1,1);];
        p = p(:);
    end

    function q=Grad2(u)
        % backward finite difference along dim 2
        u = reshape(u,n1,n2);
        q = [u(:,1) diff(u,1,2)];
        q = q(:);
    end

    function TVx=TV_MT(x)
        % Total Variation norm of x, x is a n by n matrix
        T=size(x, 2);
        grad_x=[];
        for t=1:T,
            grad_x = [grad_x Grad1(x(:,t)) Grad2(x(:,t))];
        end
        if TV_iso
            pt_sqsum = sum(grad_x.*grad_x,2);
            if TV_eps == 0; TVx = sum(sqrt(pt_sqsum)); else TVx = sum(sqrt(pt_sqsum+TV_eps)); end
        else
            TVx = norm(grad_x(:),1);
        end
    end
end