%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% This script is used to obtain the results in the paper
%%%% "Forest Sparsity for Multi-task Compressive Sensing"
%%%%% Jan. 21, 2013, Created by Chen Chen and Junzhou Huang in University
%%%%% of Texas at Arlington
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc; clear all; close all;
rand('state',0); rand('state',0);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
I = imread('pics/mri.png');

I=double(I);

I=imresize(I,[256,256]);
f=I(:);
iter=0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sigma1 = 0.01;     % acquired data noise
beta = 3.5e-2; 
d = 4.4;
maxIter=300; %X=squeeze(X); X=double(X);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[m,n]=size(I);	N=m*n;
alpha = 1e-10;
NormI = sumofsquare(f,1);
for coils=2:2:8

T=coils;

MapW=sensitivity_map([m,n],coils); %Sensitivity Map
   


for t=1:T,
    temp=imresize(I.*MapW(:,:,t), [m n]);
    temp=im256(temp);
    F(:,:,t)=temp;
end

[m n, T] = size(F); N = m*n; f=F(:); pn=m*n;

DecLevel=4;
[IdxParent, IdxChildren, Ms] = WaveTreeStructure2D(m,DecLevel);
[G,Gmat,GmatX,groups]=GroupMatrics(IdxParent);
input.G=G;input.Gt=G';input.GtG=G'*G;
input.groups=groups;input.Gmat=Gmat;

wav = daubcqf(2);%W = @(x) midwt(x,wav,DecLevel);WT = @(x) mdwt(x,wav,DecLevel);
WT = @(x) reshape(mdwt(reshape(x,m,n),wav,DecLevel),N,1);
W = @(x) reshape(midwt(reshape(x,m,n),wav,DecLevel),N,1);
Phi= A_operator(@(x) WT(x), @(x) W(x));    % notice Phi=WT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Preparing the data and operators
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
iter=iter+1;

clear OMEGA A RR b bb;

for t=1:T,
     [OMEGA{t}] = RandMask_rect(double(m/d),double(n/d),m,n);
     k(t) = 2*length(OMEGA{t})+1;
end
for t=1:T,
     start=1+sum(k(1:t-1)); stop=sum(k(1:t));
     sz(t,:)=[start, stop, pn];
end

%%%%%  Prepare for SPGL1
for t=1:T,
     f2=F(:,:, t); x0((t-1)*pn+1:t*pn,1)=WT(f2(:)); 
     f0((t-1)*pn+1:t*pn,1)=f2(:);
     R{t} = @(x) A_fhp_rect(x, OMEGA{t}, m, n);
     RT{t} = @(x) At_fhp_rect(x, OMEGA{t}, m, n);
     AO{t} = @(x) R{t}(W(x)); AOT{t} = @(x) WT(RT{t}(x)); 
     A{t} = A_operator(@(x) AO{t}(x), @(x) AOT{t}(x));
     sigma = 0.01; noise{t} = sigma*randn(k(t),1); 
     start=sz(t,1); stop=sz(t,2);
     b(start:stop,1) = A{t}* x0((t-1)*pn+1:t*pn,1)+ noise{t};
end

%%%%%  Prepare for FCSA
for t=1:T,
     f2=F(:,:, t); ff0(:,t)=f2(:); xs(:,t)=WT(ff0(:,t));
     RR{t} = A_operator(@(x) R{t}(x), @(x) RT{t}(x));
     bb{t} = RR{t}*ff0(:,t) + noise{t};
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% SPGL1_MT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
fprintf('calling the function SPGL1_MT.....\n');
opts = spgSetParms('verbosity',0);
opts.Phi=Phi;opts.f0=f0; opts.sz=sz; opts.iterations=maxIter;
nGroups = pn; temp=[1:pn]';temp=repmat(temp, [1, T]);groups=temp(:);

[x,r,g,info] = spg_mt(A,b,groups,0,opts);

for t=1:T
     index = (1:N )+ (t-1)*N;
%     index = (1:T:T*N )+ t-1;
    X1(:,t) = Phi'*x(index);
end
Y1 = sumofsquare(X1,1);
FSNR1(iter) = snr(Y1,NormI);

Trace_Time1=info.Trace_Time; Trace_SNR1=info.Trace_SNR;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% FISTA_MT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
input.f=f; input.xs=xs; input.n1=m;input.n2=n;
input.alpha=alpha;input.beta=beta;
input.Phi=Phi;input.L=1;
input.maxitr=500;input.maxiitr= 500;
input.stopCriterion=2; 
input.tolA=1e-8; input.no=maxIter; %%%%  Change no for different iteration numbers
input.l=-inf; input.u=inf; 
fprintf('calling the function FISTA_MT.....\n');
input1=input; 
input1.alpha=alpha*sqrt(T);input1.beta=beta*sqrt(T);
input1.funv=0;

out = FISTA_MT(bb, RR, ff0, input1);
X2 = out.y;
Y2 = sumofsquare(X2,1);
FSNR2(iter) = snr(Y2,NormI);
k0=(k-1)/2;
fprintf('Iter_time = %.2fsec, Iteration Num = %d, snr = %f, samp.ratio = %f \n',out.xtime(end), input.no, out.snr(end), mean(k0)/(m*n));
Trace_Time2=out.xtime'; Trace_SNR2=out.snr';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% FISTA_Forest
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
input.lamda=beta*0.5;
fprintf('calling the function FISTA_Forest.....\n');
input1=input; 
input1.alpha=alpha*sqrt(T);input1.beta=beta*sqrt(T);
input1.funv=0;

out = FISTA_Forest(bb, RR, ff0, input1);
X3 = out.y;
Y3 = sumofsquare(X3,1);
FSNR3(iter) = snr(Y3,NormI);
k0=(k-1)/2;
fprintf('Iter_time = %.2fsec, Iteration Num = %d, snr = %f, samp.ratio = %f \n',out.xtime(end), input.no, out.snr(end), mean(k0)/(m*n));
Trace_Time3=out.xtime'; Trace_SNR3=out.snr';

SNR(1,iter)=Trace_SNR1(end); SNR(2, iter)=Trace_SNR2(end);SNR(3, iter)=Trace_SNR3(end);

ls=2; ms=1; ts=16;

figure; hold on;
plot(Trace_Time1, Trace_SNR1, 'b*-.', 'linewidth',ls, 'markersize', ms);
plot(Trace_Time2, Trace_SNR2, 'kd-', 'linewidth',ls, 'markersize', ms);
plot(Trace_Time3, Trace_SNR3, 'rd-', 'linewidth',ls, 'markersize', ms);
xlabel('CPU Time (s)');ylabel('SNR');box on;
legend('SPGL1','FISTA\_MT','FISTA\_Forest', 'Location','SouthEast');
textobj = findobj('type', 'text'); set(textobj, 'fontsize', ts);
h_xlabel = get(gca,'XLabel'); set(h_xlabel,'FontSize',ts); 
h_xlabel = get(gca,'YLabel');set(h_xlabel,'FontSize',ts); 

end
CC= 2:2:8;

figure; hold on;
plot(CC,FSNR1, 'b*-.', 'linewidth',ls, 'markersize', ms);
plot(CC,FSNR2, 'kd-', 'linewidth',ls, 'markersize', ms);
plot(CC,FSNR3, 'rd-', 'linewidth',ls, 'markersize', ms);
xlabel('Iteration');ylabel('FSNR');box on;
legend('SPGL1','FISTA\_MT','FISTA\_Forest', 'Location','SouthEast');
textobj = findobj('type', 'text'); set(textobj, 'fontsize', ts);
h_xlabel = get(gca,'XLabel'); set(h_xlabel,'FontSize',ts); 
h_xlabel = get(gca,'YLabel');set(h_xlabel,'FontSize',ts); 

fprintf('Number of coils, 2, 4, 6, 8 \n');
fprintf('SPGL1(step 1), %.2f,  %.2f,  %.2f,  %.2f \n',SNR(1,1),SNR(1,2),SNR(1,3),SNR(1,4));
fprintf('FISTA_Joint (step 1), %.2f, %.2f, %.2f, %.2f \n', SNR(2,1),SNR(2,2),SNR(2,3),SNR(2,4));
fprintf('FISTA_Forest(step 2) , %.2f,  %.2f,  %.2f,  %.2f \n',SNR(3,1),SNR(3,2),SNR(3,3),SNR(3,4));
fprintf('SPGL1(step 2), %.2f,  %.2f,  %.2f,  %.2f \n',FSNR1(1),FSNR1(2),FSNR1(3),FSNR1(4));
fprintf('FISTA_Joint (step 2),%.2f,  %.2f,  %.2f,  %.2f \n',FSNR2(1),FSNR2(2),FSNR2(3),FSNR2(4));
fprintf('FISTA_Forest(step 2) , %.2f,  %.2f,  %.2f,  %.2f \n',FSNR3(1),FSNR3(2),FSNR3(3),FSNR3(4));
